-- Simple CMD Buffer - Created by MIDIMan

-- 2.2.14 prevents the "wait" command from being run in a SOC script,
-- so I created a different system for delaying cecho commands.

-- This table contains all of the "scripts" for each map.
local CMD_SCRIPTS = {
	[1] = { -- This is the map number that these "scripts" are assigned to.
		[1] = { -- This is the "script" number, determined by Argument 1 in Ultimate Zone Builder.
			-- "Scripts" are defined as a set of tables.
			{ -- Each table should have one or two elements:
				"cecho \"This is a test for this system.\"", -- The first element should be a string containing the commands to run.
				105 -- The second element can be the delay, in tics, until the next command(s) is/are run. This can be omitted if a delay is not desired.
			},
			{"cecho \"It seems to work\\pretty well!\"", 105}
		},
		[2] = {
			{"cecho \"Changing music to FDEMO1\"; tunes FDEMO1", 70}, -- You can run multiple commands in the first element by separating them with ";".
			{"cecho \"Success!\"", 70} -- Please note that backslashes (used for line breaks) and quotes must be escaped with a backslash "\".
		},
		[3] = {
			{"cecho \"DON'T MOVE\"", 105},
			{"cecho \"Turning on devmode...\"; devmode 1", 70},
			{"cecho \"Changing color to volcanic...\"; color volcanic", 70},
			{"cecho \"Changing skin to Knuckles...\"; skin knuckles", 70},
			{"cecho \"Quitting the game\\in three seconds...\"", 105},
			{"quit"}
		}
	},
	[2] = { -- This is the map number that these "scripts" are assigned to.
		[1] = { -- This is the "script" number, determined by the control linedef's front X offset in Zone Builder.
			{"cecho \"This is a test for this system.\"", 105},
			{"cecho \"It seems to work\\pretty well!\"", 105}
		},
		[2] = {
			{"cecho \"Changing music to FDEMO1\"; tunes FDEMO1", 70},
			{"cecho \"Success!\"", 70}
		},
		[3] = {
			{"cecho \"DON'T MOVE\"", 105},
			{"cecho \"Turning on devmode...\"; devmode 1", 70},
			{"cecho \"Changing color to volcanic...\"; color volcanic", 70},
			{"cecho \"Changing skin to Knuckles...\"; skin knuckles", 70},
			{"cecho \"Quitting the game\\in three seconds...\"", 105},
			{"quit"}
		}
	}
}

local CMD_BUFFER = {}
local CMD_TIMER = 0

addHook("LinedefExecute", function(line)
	if not (line and line.valid) then return end
	
	if not CMD_SCRIPTS[gamemap] then
		print("WARNING: Script for map "..gamemap.." with arg0 "..line.args[0].." not found!")
		return
	end
	
	-- Buffer doesn't get cleared between scripts, so don't clear it
-- 	CMD_BUFFER = {}

	local scriptNum = 0
	
	-- Check for Binary maps first
	if not udmf then
		if line.frontside and line.frontside.valid then -- Sanity check
			scriptNum = line.frontside.textureoffset/FRACUNIT
		else
			print("WARNING: Control linedef "..#line.." somehow doesn't have a front sidedef! Can't determine script number!")
			return
		end
	else
		scriptNum = line.args[0]
	end
	
	if not CMD_SCRIPTS[gamemap][scriptNum] then
		print("WARNING: Script for map "..gamemap.." with number "..scriptNum.." not found!")
	end
	
	-- Insert the commands into the buffer
	local i = #CMD_SCRIPTS[gamemap][scriptNum]
	while i > 0 do
		table.insert(CMD_BUFFER, 1, CMD_SCRIPTS[gamemap][scriptNum][i])
		i = $-1
	end
end, "CMDSCRIPT")

-- Reset the buffer every time the map is changed
addHook("MapChange", function()
	CMD_BUFFER = {}
	CMD_TIMER = 0
end)

-- Run the buffer
addHook("ThinkFrame", function()
	if CMD_TIMER then
		CMD_TIMER = $-1
		return
	end
	
	while CMD_BUFFER[1] do
		if CMD_BUFFER[1][1] then
			-- TODO: Make the quit command not quit on netgames?
			if consoleplayer and consoleplayer.valid then
				COM_BufInsertText(consoleplayer, CMD_BUFFER[1][1])
			elseif server and server.valid then
				COM_BufInsertText(server, CMD_BUFFER[1][1])
			end
		end
		CMD_TIMER = CMD_BUFFER[1][2] or 0
		
		table.remove(CMD_BUFFER, 1)
		
		if CMD_TIMER then
			CMD_TIMER = $-2 -- This would be 1, but the timing becomes off
			break
		end
	end
end)

-- Make sure the command buffer is properly synced
addHook("NetVars", function(net)
	CMD_BUFFER = net($)
	CMD_TIMER = net($)
end)
